# -*- coding: utf-8 -*-
"""Final IST691 GenreClassifier.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1HK1PbxXaRktIKw53_-PZxccw2FpaIPCv

Music Genre Classification<br/>
IST 691 - Group 6<br/>
Mike DeMaria, Noah Goldie, Wei Liao, Nien Tzu Shih

# Supervisied ML - 30 Second Samples

30 Second Samples
"""

import pandas as pd
!wget -np -A "*.csv" https://nand.net/mdemaria/ist691/Data/features_30_sec.csv

df = pd.read_csv('./features_30_sec.csv')
df.head(5)

"""Setup"""

df.info()

from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, classification_report

X = df.drop(['filename', 'label'], axis=1)
y = df['label']

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2)

"""Logistic Regression"""

from sklearn.linear_model import LogisticRegression
model1 = LogisticRegression()
model1.fit(X_train, y_train)
y_pred = model1.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Decision Tree"""

from sklearn.tree import DecisionTreeClassifier

model2 = DecisionTreeClassifier()
model2.fit(X_train, y_train)

y_pred = model2.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Random Forest

Is this overfitting?
"""

from sklearn.ensemble import RandomForestClassifier
model3 = RandomForestClassifier(n_estimators=50)
model3.fit(X_train, y_train)

y_pred = model3.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Support Vector Machine (SVM)"""

from sklearn.svm import SVC

model4 = SVC()
model4.fit(X_train, y_train)

y_pred = model4.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""KNN"""

from sklearn.neighbors import KNeighborsClassifier
model5 = KNeighborsClassifier(n_neighbors=10)
model5.fit(X_train, y_train)

y_pred = model5.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""# Supervisied ML - 3 Second Samples

Setup
"""

!wget -np -A "*.csv" https://nand.net/mdemaria/ist691/Data/features_3_sec.csv
df2 = pd.read_csv('./features_3_sec.csv')
df2.head(5)

"""Logistic Regression"""

from sklearn.linear_model import LogisticRegression
model1 = LogisticRegression()
model1.fit(X_train, y_train)
y_pred = model1.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Decision Tree"""

from sklearn.tree import DecisionTreeClassifier

model2 = DecisionTreeClassifier()
model2.fit(X_train, y_train)

y_pred = model2.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Random Forest Classifier"""

from sklearn.ensemble import RandomForestClassifier
model3 = RandomForestClassifier(n_estimators=50)
model3.fit(X_train, y_train)

y_pred = model3.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""Support Vector Machine (SVM)"""

from sklearn.svm import SVC

model4 = SVC()
model4.fit(X_train, y_train)

y_pred = model4.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""KNN"""

from sklearn.neighbors import KNeighborsClassifier
model5 = KNeighborsClassifier(n_neighbors=10)
model5.fit(X_train, y_train)

y_pred = model5.predict(X_test)

# Evaluate the model
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy: {accuracy}")
print("Classification Report:")
print(classification_report(y_test, y_pred))

"""# Basic ANN as Benchmark"""

import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler, LabelEncoder
from sklearn.metrics import accuracy_score
from keras.models import Sequential
from keras.layers import Dense
from google.colab.drive import mount

!wget -np -A "*.csv" https://nand.net/mdemaria/ist691/Data/features_30_sec.csv

# Replace this filepath with your own
df = pd.read_csv('./features_30_sec.csv')

df.head()

print(df.dtypes)

"""Feed Forward Network"""

df = df.drop('filename', axis=1)
X = df.drop('label', axis=1)
y = df['label']

# Encode the labels for multi-class classification
label_encoder = LabelEncoder()
y = label_encoder.fit_transform(y)

# Split the data into training and testing sets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Standardize the features (mean=0 and variance=1)
scaler = StandardScaler()
X_train = scaler.fit_transform(X_train)
X_test = scaler.transform(X_test)

# Number of classes
num_classes = len(label_encoder.classes_)

# Build the model
model = Sequential()
model.add(Dense(128, input_dim=X_train.shape[1], activation='relu'))
model.add(Dense(64, activation='relu'))
model.add(Dense(32, activation='relu'))
model.add(Dense(num_classes, activation='softmax'))  # Softmax for multi-class classification

# Compile the model
model.compile(loss='sparse_categorical_crossentropy', optimizer='adam', metrics=['accuracy'])

# Train the model
model.fit(X_train, y_train, epochs=10, batch_size=32, validation_split=0.2)

# Evaluate the model on the test set
y_pred_probs = model.predict(X_test)
y_pred = np.argmax(y_pred_probs, axis=1)
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy on the test set: {accuracy}")

"""# Spectrogram Generation"""

import os
import librosa
import librosa.display
import matplotlib.pyplot as plt
import numpy as np
!pip install pydub
from pydub import AudioSegment
from pydub.utils import make_chunks

# This block is used to split the 30 second waveforms into three 10 second waveforms,
# and to generate the mel spectrographs.  As we've converted and saved the files,
# you don't need to do this again, since you can download all the audio and images
# at https://nand.net/mdemaria/ist691/Data/

# http://man.hubwiz.com/docset/LibROSA.docset/Contents/Resources/Documents/generated/librosa.display.specshow.html
# STFT power spectrum
# Similar to set in GTZAN, with Hz on the y axis
def generate_image_STFTPowerSpectrum(readpath, writepath):
	y, sr = librosa.load(readpath)

	spec = np.abs(librosa.stft(y, hop_length=512))
	spec = librosa.amplitude_to_db(spec, ref=np.max)
	librosa.display.specshow(spec, sr=sr );

	plt.savefig(writepath)

# Chromagram with pitch class
# y axis is based on pitch
def generate_image_Chromagram(readpath, writepath):
	y, sr = librosa.load(readpath)

	C = librosa.feature.chroma_cqt(y=y, sr=sr)
	librosa.display.specshow(C)

	plt.savefig(writepath)


# Function to process one genre
def process_genre(split10, genre):
	directory_source = './genres_original/' + genre;
	directory_dest_sftp = './images_SFPT/'
	directory_dest_pitch = './images_Pitch/'

	if(split10 == True):
		directory_source = './genres_10sec/' + genre;
		directory_dest_sftp = './images_10sec_SFPT/'
		directory_dest_pitch = './images_10sec_Pitch/'


	for filename in os.listdir(directory_source):
		read_file_path = os.path.join(directory_source, filename)
		save_file_path_SFTP = directory_dest_sftp + genre + '/' + filename[:-3] + 'png';
		save_file_path_Pitch = directory_dest_pitch + genre + '/' + filename[:-3] + 'png';
		print('read: ' + read_file_path)
		generate_image_STFTPowerSpectrum(read_file_path, save_file_path_SFTP)
		print('save: ' + save_file_path_SFTP)
		generate_image_Chromagram(read_file_path, save_file_path_Pitch)
		print('save: ' + save_file_path_Pitch)


# Split an audio file
# https://stackoverflow.com/questions/36799902/how-to-splice-an-audio-file-wav-format-into-1-sec-splices-in-python?rq=1
# It looks like in a few small cases, it'll split another very small file, probably due to rounding error
# So if we're splitting by 3, any {name}3.wav file should be deleted afterwards
def split_song(genre):
	directory_source = './genres_original/' + genre;
	for filename in os.listdir(directory_source):
		read_file_path = os.path.join(directory_source, filename)
		print(read_file_path)
		myaudio = AudioSegment.from_file(read_file_path , "wav")
		chunk_length_ms = (myaudio.duration_seconds / 3) * 1000 # Cut into 3 chunks
		chunks = make_chunks(myaudio, chunk_length_ms) #Make chunks of one sec
		for i, chunk in enumerate(chunks):
			chunk_name = "./genres_10sec/" + genre + "/" + filename + "{0}.wav".format(i)
			chunk.export(chunk_name, format="wav")


# Function to process all genres
# It seems like Python doesn't deallocate memory as it processes through,
# so the memory usage keeps going up and up.  You may need to run these individually instead.
def process_all_genres(split10):
	process_genre(split10, 'blues')
	process_genre(split10, 'classical')
	process_genre(split10, 'country')
	process_genre(split10, 'disco')
	process_genre(split10, 'hiphop')
	process_genre(split10, 'jazz')
	process_genre(split10, 'metal')
	process_genre(split10, 'pop')
	process_genre(split10, 'reggae')
	process_genre(split10, 'rock')

# Function to split all genres into 10 second chunks
def split_all_genres():
	split_song('blues')
	split_song('classical')
	split_song('country')
	split_song('disco')
	split_song('hiphop')
	split_song('jazz')
	split_song('metal')
	split_song('pop')
	split_song('reggae')
	split_song('rock')

# Process individually
#split10 = True
#process_genre(split10, 'blues')
#process_genre(split10, 'classical')
#process_genre(split10, 'country')
#process_genre(split10, 'disco')
#process_genre(split10, 'hiphop')
#process_genre(split10, 'jazz')
#process_genre(split10, 'metal')
#process_genre(split10, 'pop')
#process_genre(split10, 'reggae')
#process_genre(split10, 'rock')

# Process everything.  Comment this line out to *not* convert/split again.
# Given the memory issue mentioned above, this might not be a great idea.
#split_all_genres()
#process_all_genres(False)
#process_all_genres(True)

"""# CNN Model"""

!wget -r -np -A "*.png" https://nand.net/mdemaria/ist691/Data/images_Pitch/
!wget -r -np -A "*.png" https://nand.net/mdemaria/ist691/Data/testing/test_images_Pitch/

import pandas as pd
import tensorflow as tf
import matplotlib.pyplot as plt
import numpy as np
import time
import os
import keras
from tensorflow import keras
from keras.utils import get_file
from keras import preprocessing
from keras import layers
from keras import optimizers
from keras import activations
from tensorflow.keras.preprocessing import image
from tensorflow.keras.applications.mobilenet import preprocess_input

"""Some extra functions to evaluate the models"""

def plot_training(history):
  fig, ax = plt.subplots(3, 1, figsize=(6,10))
  n_epochs = len(history.history['loss'])
  min_loss = min(history.history['val_loss'])
  max_loss = max(history.history['loss'])
  min_acc = min(history.history['val_accuracy'])
  max_acc = max(history.history['val_accuracy'])
  min_3acc = min(history.history['val_top_k_categorical_accuracy'])
  max_3acc = max(history.history['val_top_k_categorical_accuracy'])

  # Loss Graph
  ax[0].plot(history.history['loss'])
  ax[0].plot(history.history['val_loss'])
  ax[0].set_xlabel('Epoch')
  ax[0].set_ylabel('Loss')
  ax[0].annotate('Minimum Val Loss: ' + str(round(min_loss, 3)), xy = (n_epochs/2, max_loss - 0.1))
  # Accuracy Graph
  ax[1].plot(history.history['accuracy'])
  ax[1].plot(history.history['val_accuracy'])
  ax[1].set_ylim((0,1))
  ax[1].set_xlabel('Epoch')
  ax[1].set_ylabel('Accuracy')
  ax[1].annotate('Maximum Val Accuracy: ' + str(round(max_acc, 3)), xy = (n_epochs/2, 0.05))
  # Top 3 Accuracy Graph
  ax[2].plot(history.history['top_k_categorical_accuracy'])
  ax[2].plot(history.history['val_top_k_categorical_accuracy'])
  ax[2].set_ylim((0,1))
  ax[2].set_xlabel('Epoch')
  ax[2].set_ylabel('Top 3 Accuracy')
  ax[2].annotate('Maximum Val Top 3 Accuracy: ' + str(round(max_3acc, 3)), xy = (n_epochs/3, 0.05))

  fig.suptitle('Model Evaluation Metrics')
  fig.legend(['Training', 'Validation'], loc = 'upper right')

def confint(p, n):
  # Standard error of a binomial distribution, based on probability and sample size
  e = np.sqrt(p*(1-p)/n)
  # Calculating lower bound and upper bound based on z-score of 1.96 (95% confidence interval)
  lb = round(p - 1.96*e, 3)
  ub = round(p + 1.96*e, 3)
  return (lb, ub)

image_directory = './nand.net/mdemaria/ist691/Data/images_Pitch'
image_size = (480, 640)
batch_size = 32

train = preprocessing.image_dataset_from_directory(directory = image_directory,
                                                   label_mode = 'categorical',
                                                   validation_split = 0.2,
                                                   subset = 'training',
                                                   seed = 6910,
                                                   image_size = image_size,
                                                   batch_size = batch_size)

val = preprocessing.image_dataset_from_directory(directory = image_directory,
                                                 label_mode = 'categorical',
                                                 validation_split = 0.2,
                                                 subset = 'validation',
                                                 seed = 6910,
                                                 image_size = image_size,
                                                 batch_size = batch_size)

train = train.prefetch(buffer_size = 32)
val = val.prefetch(buffer_size = 32)


model6 = keras.Sequential(
    [
        keras.Input(shape=(480, 640, 3), batch_size = batch_size),
        layers.Cropping2D(cropping = ((55,52),(78,63))),

        layers.Conv2D(filters = 32, kernel_size = (9,9), padding = 'same'),
        layers.MaxPool2D((2,2)),
        layers.BatchNormalization(),
        layers.Activation('relu'),

        layers.Conv2D(filters = 64, kernel_size = (7, 7), padding = 'same'),
        layers.MaxPool2D((2,2)),
        layers.BatchNormalization(),
        layers.Activation('relu'),

        layers.Conv2D(filters = 64, kernel_size = (5,5), padding = 'same'),
        layers.MaxPool2D((2,2)),
        layers.BatchNormalization(),
        layers.Activation('relu'),

        layers.Conv2D(filters = 16, kernel_size = (1,1), padding = 'same'),
        layers.MaxPool2D((2,2)),
        layers.BatchNormalization(),
        layers.Activation('relu'),

        layers.Flatten(),
        layers.Dropout(0.75),

        layers.Dense(256, activation = 'elu'),
        layers.Dropout(0.5),

        layers.Dense(128, activation = 'elu'),
        layers.Dropout(0.5),

        layers.Dense(64, activation = 'elu'),
        layers.Dropout(0.5),

        layers.Dense(32, activation = 'elu'),
        layers.Dropout(0.25),

        layers.Dense(10, activation = 'softmax')
    ]
    )

model6.summary()


learning_rate = 0.00005
epochs = 200

top_3_accuracy = keras.metrics.TopKCategoricalAccuracy(k = 3)

# legacy Adam used as its faster on M1 macs
model6.compile(optimizer = optimizers.legacy.Adam(learning_rate = learning_rate),
               loss = 'categorical_crossentropy',
               metrics = ['accuracy', top_3_accuracy],)


#https://stackoverflow.com/questions/48285129/saving-best-model-in-keras
#from keras.callbacks import EarlyStopping
#from keras.callbacks import ModelCheckpoint
#from keras.callbacks import ReduceLROnPlateau
#earlyStopping = EarlyStopping(monitor='val_accuracy', patience=10, verbose=0, mode='min')
#mcp_save = ModelCheckpoint('.mdl_wts.hdf5', save_best_only=True, monitor='val_accuracy', mode='min')
#reduce_lr_loss = ReduceLROnPlateau(monitor='val_accuracy', factor=0.1, patience=7, verbose=1, epsilon=1e-4, mode='min')

#model6_history = model6.fit(train, epochs = epochs, validation_data = val, callbacks=[earlyStopping, mcp_save, reduce_lr_loss])
model6_history = model6.fit(train, epochs = epochs, validation_data = val)

plot_training(model6_history)

print("Model 6b Confidence Intervals",
      '\nAccuracy:', confint(max(model6_history.history['val_accuracy']), 199),
      '\nMaximum Top 3 Accuracy:', confint(max(model6_history.history['val_top_k_categorical_accuracy']), 199))


model6.save('./model6_colab_200Pitch')

#Model 6b Confidence Intervals
#Accuracy: (0.651, 0.776)
#Maximum Top 3 Accuracy: (0.87, 0.949)

!tar -cf model6_colab_200Pitch.tar model6_colab_200Pitch

"""# Predictions"""

import numpy as np
import pandas as pd
from pandas import read_csv

!wget -np -A "*.csv" https://nand.net/mdemaria/ist691/Data/rym_clean1.csv

d = read_csv('rym_clean1.csv')

df = d.values

recommended_by_genre_list = pd.DataFrame()

for line in df:
  process = {}
  process['album_name'] = line[2]
  process['artist'] = line[3]
  process['avg_rating'] = line[9]
  process['rating_count'] = line[10]
  process['review_count'] = line[11]
  process['pblues'] = False
  process['pclassical'] = False
  process['pcountry'] = False
  process['pdisco'] = False
  process['phiphop'] = False
  process['pjazz'] = False
  process['pmetal'] = False
  process['ppop'] = False
  process['preggae'] = False
  process['prock'] = False
  process['sblues'] = False
  process['sclassical'] = False
  process['scountry'] = False
  process['sdisco'] = False
  process['shiphop'] = False
  process['sjazz'] = False
  process['smetal'] = False
  process['spop'] = False
  process['sreggae'] = False
  process['srock'] = False

  # Process Primary Genres
  if(isinstance(line[6], str)):
    if "Blues" in line[6]:
      process['pblues'] = True
    if "Orchestral" in line[6] or "Symphony" in line[6] or "Modern Classical" in line[6]:
      process['pclassical'] = True
    if "Country" in line[6]:
      process['pcountry'] = True
    if "Disco" in line[6] or "Dance" in line[6]:
      process['pdisco'] = True
    if "Hip Hop" in line[6]:
      process['phiphop'] = True
    if "Jazz" in line[6] or "Big Band" in line[6]:
      process['pjazz'] = True
    if "Metal" in line[6]:
      process['pmetal'] = True
    if "Adult Contemporary" in line[6] or "Pop" in line[6]:
      process['ppop'] = True
    if "Reggae" in line[6]:
      process['preggae'] = True
    if "Rock" in line[6]:
      process['prock'] = True

 # Process Secondary Genres
  if(isinstance(line[7], str)):
    if "Blues" in line[7]:
      process['sblues'] = True
    if "Orchestral" in line[7] or "Symphony" in line[7] or "Modern Classical" in line[7]:
      process['sclassical'] = True
    if "Country" in line[7]:
      process['scountry'] = True
    if "Disco" in line[7]:
      process['sdisco'] = True
    if "Hip Hop" in line[7]:
      process['shiphop'] = True
    if "Jazz" in line[7] or "Big Band" in line[7]:
      process['sjazz'] = True
    if "Metal" in line[7]:
      process['smetal'] = True
    if "Adult Contemporary" in line[7] or "Pop" in line[7]:
      process['spop'] = True
    if "Reggae" in line[7]:
      process['sreggae'] = True
    if "Rock" in line[7]:
      process['srock'] = True

  df_process = pd.DataFrame([process])
  recommended_by_genre_list = pd.concat([recommended_by_genre_list, df_process], ignore_index=True)


def find_primary(genre):
  if genre == "clues":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pblues'] == True]
  if genre == "classical":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pclassical'] == True]
  if genre == "country":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pcountry'] == True]
  if genre == "disco":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pdisco'] == True]
  if genre == "hiphop":
    return recommended_by_genre_list.loc[recommended_by_genre_list['phiphop'] == True]
  if genre == "jazz":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pjazz'] == True]
  if genre == "metal":
    return recommended_by_genre_list.loc[recommended_by_genre_list['pmetal'] == True]
  if genre == "pop":
    return recommended_by_genre_list.loc[recommended_by_genre_list['ppop'] == True]
  if genre == "reggae":
    return recommended_by_genre_list.loc[recommended_by_genre_list['preggae'] == True]
  if genre == "rock":
    return recommended_by_genre_list.loc[recommended_by_genre_list['prock'] == True]

def find_secondary(genre):
  if genre == "clues":
    return recommended_by_genre_list.loc[recommended_by_genre_list['sblues'] == True]
  if genre == "classical":
    return recommended_by_genre_list.loc[recommended_by_genre_list['sclassical'] == True]
  if genre == "country":
    return recommended_by_genre_list.loc[recommended_by_genre_list['scountry'] == True]
  if genre == "disco":
    return recommended_by_genre_list.loc[recommended_by_genre_list['sdisco'] == True]
  if genre == "hiphop":
    return recommended_by_genre_list.loc[recommended_by_genre_list['shiphop'] == True]
  if genre == "jazz":
    return recommended_by_genre_list.loc[recommended_by_genre_list['sjazz'] == True]
  if genre == "metal":
    return recommended_by_genre_list.loc[recommended_by_genre_list['smetal'] == True]
  if genre == "pop":
    return recommended_by_genre_list.loc[recommended_by_genre_list['spop'] == True]
  if genre == "reggae":
    return recommended_by_genre_list.loc[recommended_by_genre_list['sreggae'] == True]
  if genre == "rock":
    return recommended_by_genre_list.loc[recommended_by_genre_list['srock'] == True]

# Recommendations will return up to 9 albums:
#   the highest rated album in a genre
#   the album with the most ratings in a genre
#   the album with the most reviews in a genre
# It will pull from the primary genres for the first argument, primary genres for the second argument,
# and secondary genres for the third argument.  It will automatically de-dup, so you should not get the
# same album listed twice.
def build_recommendation(primary, secondary, teritary):
  recommendations = ""
  primary_by_ratingscore = primary.loc[primary['avg_rating'].idxmax()]
  primary_by_ratingcount = primary.loc[primary['rating_count'].idxmax()]
  primary_by_reviewcount = primary.loc[primary['review_count'].idxmax()]
  secondary_by_ratingscore = secondary.loc[secondary['avg_rating'].idxmax()]
  secondary_by_ratingcount = secondary.loc[secondary['rating_count'].idxmax()]
  secondary_by_reviewcount = secondary.loc[secondary['review_count'].idxmax()]
  teritary_by_ratingscore = teritary.loc[teritary['avg_rating'].idxmax()]
  teritary_by_ratingcount = teritary.loc[teritary['rating_count'].idxmax()]
  teritary_by_reviewcount = teritary.loc[teritary['review_count'].idxmax()]


  recommendations += "If you like this song, give these albums a try\n"
  prec1 = "\t" + primary_by_ratingscore['album_name'] + " by " + primary_by_ratingscore['artist'] + "\n"
  prec2 = "\t" + primary_by_ratingcount['album_name'] + " by " + primary_by_ratingcount['artist'] + "\n"
  prec3 = "\t" + primary_by_reviewcount['album_name'] + " by " + primary_by_reviewcount['artist'] + "\n"

  recommendations += prec1
  if(prec2 != prec1):
    recommendations += prec2
  if(prec3 != prec2 and prec3 != prec1):
    recommendations += prec3

  srec1 = "\t" + secondary_by_ratingscore['album_name'] + " by " + secondary_by_ratingscore['artist'] + "\n"
  srec2 = "\t" + secondary_by_ratingcount['album_name'] + " by " + secondary_by_ratingcount['artist'] + "\n"
  srec3 = "\t" + secondary_by_reviewcount['album_name'] + " by " + secondary_by_reviewcount['artist'] + "\n"

  if(srec1 == prec1 or srec1 == prec2 or srec1 == prec3):
    srec1 = ''
  if(srec2 == prec1 or srec2 == prec2 or srec2 == prec3):
    srec2 = ''
  if(srec3 == prec1 or srec3 == prec2 or srec3 == prec3):
    srec3 = ''

  if(srec1 != '' or srec2 != '' or srec3 != ''):
    recommendations += "\nHere are some more options\n"

  if(srec1 != ''):
    recommendations += srec1
  if(srec2 != '' and srec2 != srec1):
    recommendations += srec2
  if(srec3 != '' and srec3 != srec2 and srec3 != srec1):
    recommendations += srec3



  trec1 = "\t" + teritary_by_ratingscore['album_name'] + " by " + teritary_by_ratingscore['artist'] + "\n"
  trec2 = "\t" + teritary_by_ratingcount['album_name'] + " by " + teritary_by_ratingcount['artist'] + "\n"
  trec3 = "\t" + teritary_by_reviewcount['album_name'] + " by " + teritary_by_reviewcount['artist'] + "\n"

  if(trec1 == prec1 or trec1 == prec2 or trec1 == prec3 or trec1 == srec1 or trec1 == srec2 or trec1 == srec3):
    trec1 = ''
  if(trec2 == prec1 or trec2 == prec2 or trec2 == prec3 or trec2 == srec1 or trec2 == srec2 or trec2 == srec3):
    trec2 = ''
  if(trec3 == prec1 or trec3 == prec2 or trec3 == prec3 or trec3 == srec1 or trec3 == srec2 or trec3 == srec3):
    trec3 = ''

  if(trec1 != '' or trec2 != '' or trec3 != ''):
    recommendations += "\nHere are some deeper cuts\n"

  if(trec1 != ''):
    recommendations += trec1
  if(trec2 != '' and trec2 != trec1):
    recommendations += trec2
  if(trec3 != '' and trec3 != trec2 and trec3 != trec1):
    recommendations += trec3



  return recommendations


# The interface into this code block.  We're accepting two genres as input.
def make_recommendation(primary, secondary):
	print(build_recommendation(find_primary(primary),find_primary(secondary),find_secondary(primary)))

import numpy as np
import pandas as pd
import keras
from tensorflow import keras
from tensorflow.keras.preprocessing import image
from tensorflow.keras.applications.mobilenet import preprocess_input

# Retrieve data and load model
# As we're using a saved model here, you do not need to re-train the CNN to run this part
!wget -r -np -A "*.png" https://nand.net/mdemaria/ist691/Data/testing/test_images_Pitch/
!wget -np -A "*.tar" https://nand.net/mdemaria/ist691/Data/model6_colab_200Pitch.tar
!tar -xvf model6_colab_200Pitch.tar
model6 = keras.models.load_model('./model6_colab_200Pitch')

class_labels = ['blues', 'classical', 'country', 'disco','hiphop','jazz','metal','pop','reggae','rock']

def predict_image(path, labelname, recommend, showfull):
  x = image.load_img(path)
  x = image.img_to_array(x)
  x = np.expand_dims(x, axis=0)
#  x = preprocess_input(x)
  predictions = model6.predict(x)
  bestoption_label = '';
  bestoption_value = 0.00;

  genre_dict = {}
  for i, label in enumerate(class_labels):
  	genre_dict[label] = predictions[0][i]

  #genre_prediction = np.sort(genre_prediction, order=[1])
  genre_dict = dict(sorted(genre_dict.items(), key=lambda item: item[1], reverse=True))
  likely_first = list(genre_dict.keys())[0]
  likely_second = list(genre_dict.keys())[1]
  likely_third = list(genre_dict.keys())[2]

  print(labelname + " is most likely " + likely_first + " (" + str(int(genre_dict[likely_first] * 100)) + "%) or " + likely_second + " (" + str(int(genre_dict[likely_second] * 100)) + "%)")

  if(showfull):
      for i, label in enumerate(class_labels):
        print(f'{label}: {predictions[0][i]}')

  if(recommend):
      make_recommendation(likely_first,likely_second)

basepath = './nand.net/mdemaria/ist691/Data/testing/test_images_Pitch/'
# First boolean outputs suggestions
# Second boolean outputs all probabilities
#predict_image(basepath + 'FurElise-Classical.png', 'FurElise-Classical', True, False)
#predict_image(basepath + 'FurElise-Disco.png', 'FurElise-Disco', True, False)
#predict_image(basepath + 'FurElise-DubStep.png', 'FurElise-DubStep', True, False)
#predict_image(basepath + 'FurElise-Jazz.png', 'FurElise-Jazz', True, True)
#predict_image(basepath + 'FurElise-JazzLowQual.png', 'FurElise-JazzLowQual', True, False)
#predict_image(basepath + 'FurElise-Metal.png', 'FurElise-Metal', True, False)
#predict_image(basepath + 'FurElise-Reggae.png', 'FurElise-Reggae', True, False)
#predict_image(basepath + 'BackInBlack.png', 'BackInBlack', True, False)
#predict_image(basepath + 'BewareOfTheBoys.png', 'BewareOfTheBoys', True, False)
#predict_image(basepath + 'CopacabanaLowQual.png', 'CopacabanaLowQual', True, False)
#predict_image(basepath + 'Money.png', 'Money', True, True)
#predict_image(basepath + 'OldTownRoad.png', 'OldTownRoad', True, False)
#predict_image(basepath + 'TunahTunahTun.png', 'TunahTunahTun', True, False)
#predict_image(basepath + 'GhostRiders.png', 'GhostRiders', True, True)
#predict_image(basepath + 'Copacabana.png', 'Copacabana', True, False)

"""Source for test clips:<br/>
Classical
1:45-2:15
https://www.youtube.com/watch?v=e4d0LOuP4Uw

Disco
0:50-1:20
https://www.youtube.com/watch?v=ZH7kD1pJ7EE

Dubstep
1:05-1:35
https://www.youtube.com/watch?v=dBb060OPegg

Reggae
0:30-1:00
https://www.youtube.com/watch?v=MBX2EDGX85o

Jazz
0:20-0:50
https://www.youtube.com/watch?v=qIPZl6-IgVM

Jazz Low Qual
0:20-0:50
https://www.youtube.com/watch?v=G9FOjvzXYGs

Metal
1:45-2:15
https://www.youtube.com/watch?v=tEWl60cUd_Y

Old Town Road
0:30-1:00
https://www.youtube.com/watch?v=r7qovpFAGrQ

Money
0:50-1:20
https://www.youtube.com/watch?v=-0kcet4aPpQ

Back in black
0:40-1:10
https://www.youtube.com/watch?v=pAgnJDJN4VA

Copacabana
4:00-4:30
https://www.youtube.com/watch?v=KRs4AUAEkHE

Copacabana Low Qual
0:20-0:50
https://www.youtube.com/watch?v=XDR247fmhnc

Beware of the boys
0:35-1:-05
https://www.youtube.com/watch?v=x9WO2ieJMYk

Tunak Tunak Tun
0:30-1:100
https://www.youtube.com/watch?v=4qRZmFYdozY

Ghost Riders In the Sky
0:30-1:00
https://www.youtube.com/watch?v=nOWjX4BpC24

"""